;                        ._____ _______ __________.
;                        |                      (_:
;                        |   PIC Repeater logic   |
;                        |_v2.0 (2002-2006 OH3HRK)|
;                        :_)________/)____________|
;                        "
;
;
;  DTMF:
;  *      - Opens the repeater as a 1750hz tone does.
;  *0     - Opens the repeater and gives the infotext (callsign, locator, whatever)paikan ja lokaattorin
;  *9     - Closes the repeater
;  #9     - Closes the repeater
;  A234   - Settings mode (you will hear "SET" in CW)
;         - * exit the settings mode
;         - 0 close the repeater for 5min
;         - 1 close the repeater for 10min
;         - 2 close the repeater for 20min
;         - 3 close the repeater for 30min
;         - 4 close the repeater for 40min
;         - 5 close the repeater for 50min
;         - 6 close the repeater for 60min
;         - 7 close the repeater for 120min
;         - 8 close the repeater for 240min
;         - 9 close the repeater for ever. (the device must be rebooted)
;         - # Give the statistics, the output is:
;             - OPN ???? ???? = QSO ???? ???? = K
;               How many times the repeater has been opened?
;               How many QSOs people have kept on the repeater (at least 5 overs)
;               The numbers are given twice. NOTE: The numbers will be in hexadecimal format!!
;               After this: * = Return to the Settings mode -- # = Clear the stats and return
;         - A Output pin RB6 (pin 12 of the IC) OFF (GND)
;         - B Output pin RB6 (pin 12 of the IC) ON (+5V)
;
;  Versiot:
;    v1.0     - The first version
;    v1.1     - Fixed a little EEPROM bug
;    v1.1i    - The squelch input has been inverted.
;    v2.0     - New hardware and the microcontroller (PIC16F84 -> PIC16F628)

;--[ compiler thingies ]--:

        title   "REPEATER_LOGIC_20"
        list    P=16F628;f=inhx8m
        include "P16F628.INC"

;--[ settings for the chip programmer ]--:
;--[ xt, watchdog, power up timer ]--:
        __config        0x3FFD

        

;--[ eeprom-settings ]--:
        org     0x2100

        ; Opening CW-text
        data    'O','H','3','R','A','C','-','-'

        ; ID CW-text (given every X minutes)
        data    'O','H','3','R','A','C','-','-'

        ; Ending CW-text
        data    'O','H','3','R','A','C','-','-'

        ; Closing CW-text (2 bytes)   18h 19h 
        data    'C','L'

        ; Maximum time people can talk, before the repeater closes 0x3C = 60 seconds
        data    0x3C ;--- 1Ah

        ; How often do we need the CW ID? 0A = 10 min
        data    0x0A

        ; CW-frequency .. a smaller value makes the tone lower --- 1Ch
        data 0x90

        ; CW-speed .. a smaller value makes it faster
        data    0x1b ;--- 1Dh

        ; Seconds of inactivity before the repeater closes 1Eh
        data 0x08

        ; After the repeater closes because of inactivity, how many
        ; seconds people must wait until the repeater REALLY closes (and you need a tone to open it) 1Fh
        data 0x07 

        ; CW info text (when opened with DTMF *0)
        data    'D','E',' ','O','H','3','R','A','C',' '
        data    'L','A','H','T','I',' ','K','P','2','0','T','X'
        data    '-','-','-','-','-','-'

        ; Counters are here: Times opened (16bit) and Number of QSOs (16bit)
        data    0x00,0x00,0x00,0x00

;--[ variables ]--:
        cblock  0x20
        IntrCount               
        Safe_W                  
        Safe_S
        Counter                 ; second counter
        Counter_L               ; second counter too 0-60
        Counter_H               ; minute counter
        CWSpeed
        CWChar
        CloseTime
        Temp
        Temp2
        Temp3
        Vuorot
        Counter2
        IDTime
        CWPitch
        ToneCount
        SpTimeOut
        SpTimeOutL
        TilastoTmp
       	endc

IO_Mute         equ     0       ; Mute pin
IO_TX           equ     1       ; TX on pin
IO_SND          equ     2       ; Sound out pin
IO_Led          equ     3       ; Green LED
IO_DTMF         equ     4       ; DTMF
IO_Sql          equ     5       ; Squelch
IO_Pin12        equ     6       ; PIN 12
IO_Tone         equ     7       ; Tone input (logic level)


;--[ start ]--:
        org     0              
        goto    Alku            ; jump over the interrupt routine

;--[ interrupt routine ]--:
        org     4               ; the interrupt begins at org 4
        movwf   Safe_W          ; W -> Safe_W
        swapf   STATUS, W       ; Swap the nibbles of STATUS and place to W
        movwf   Safe_S          ; W -> Safe_S

        clrwdt
        incf    IntrCount, F    ; Increase IntrCount on every interrupt
                                ; Is IntrCount 0xF0?
        movfw   IntrCount       ; IntrCount -> W
        xorlw   0xF0            ; W XOR F0. If W=0xF0 then the ZERO flag is set
        btfsc   STATUS, Z       ; Skip Clear. Skip if the ZERO flag was not set
        call    Sekunti         ; A one second has elapsed

        movfw   IntrCount
        xorlw   0x78
        btfsc   STATUS, Z
        bsf     PORTA, IO_Led   ; LED on

        ; Restore the STATUS and W after the interrupt

        swapf   Safe_S, W       ; Get the old STATUS from the Safe_S register. Swap.
        movwf   STATUS          ; W -> STATUS
        swapf   Safe_W, F       ; Swap the Safe_W nibbles.
        swapf   Safe_W, W       ; Swap the Safe_W nibbles again and place to W.
        bcf     INTCON, 2       ; INTCON register, interrupt flag off.
        retfie                  ; Returning from the interrupt

;--[ when a second has elapsed, increase the counter ]--:
Sekunti:
        bcf     PORTA, IO_Led   ; LED off
        clrf    IntrCount       ; Clear IntrCount
        incf    Counter, F
        incf    Counter2, F
        incf    Counter_L, F

        movfw   Counter_L
        xorlw   0x3C
        btfss   STATUS, Z
        return
        clrf    Counter_L
        incf    Counter_H, F    ; If L went over 60, increase the H
        return

;--[ CW table ]--:
;--[ empty space is indicated by 1s, until the first 0 comes ]--:
;--[ 1 = dot and 0 = dash ]--:
CWMerkki
                addwf   PCL, F

                retlw           b'11000000'     ; 0
                retlw           b'11010000'     ; 1
                retlw           b'11011000'     ; 2
                retlw           b'11011100'     ; 3
                retlw           b'11011110'     ; 4
                retlw           b'11011111'     ; 5
                retlw           b'11001111'     ; 6
                retlw           b'11000111'     ; 7
                retlw           b'11000011'     ; 8
                retlw           b'11000001'     ; 9

                retlw           b'11111010'     ; 10 - A .-
                retlw           b'11100111'     ; 11 - B -...
                retlw           b'11100101'     ; 12 - C -.-.
                retlw           b'11110011'     ; 13 - D -..
                retlw           b'11111101'     ; 14 - E .
                retlw           b'11101101'     ; 15 - F ..-.
                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU

                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU
                retlw           b'11111111'     ; RFU
                retlw           b'11010010'     ; 26 - 
                retlw           b'11101010'     ; 27 - 
                retlw           b'11100001'     ; 28 - 
                retlw           b'11001010'     ; 29 - (KA) -.-.-

                retlw           b'11001001'     ; 30 - (KN) -.--.
                retlw           b'10111010'     ; 31 - (SK) ...-.-
                retlw           b'00000000'     ; 32 - SPACE
                retlw           b'11011001'     ; 33 - !
                retlw           b'10101101'     ; 34 - "
                retlw           b'11111111'     ;(35 - #)
                retlw           b'11111111'     ;(36 - $)
                retlw           b'11111111'     ;(37 - %)
                retlw           b'11111111'     ;(38 - &)
                retlw           b'10100001'     ; 39 - '

                retlw           b'11001001'     ; 40 - (
                retlw           b'10010010'     ; 41 - )
                retlw           b'11111111'     ; 42 - *
                retlw           b'11010101'     ; 43 - + .-.-.
                retlw           b'10001100'     ; 44 - ,
                retlw           b'11111111'     ; 45 - - (EMPTY CHAR)
                retlw           b'10101010'     ; 46 - .
                retlw           b'11001101'     ; 47 - /

                retlw           b'11000000'     ; 48 - 0
                retlw           b'11010000'     ; 49 - 1
                retlw           b'11011000'     ; 50 - 2
                retlw           b'11011100'     ; 51 - 3
                retlw           b'11011110'     ; 52 - 4
                retlw           b'11011111'     ; 53 - 5
                retlw           b'11001111'     ; 54 - 6
                retlw           b'11000111'     ; 55 - 7
                retlw           b'11000011'     ; 56 - 8
                retlw           b'11000001'     ; 57 - 9

                retlw           b'10000111'     ; 58 - :
                retlw           b'11111111'     ;(59 - ;)
                retlw           b'11111111'     ;(60 - <)
                retlw           b'11001110'     ; 61 - =
                retlw           b'11111111'     ;(62 - >)
                retlw           b'10110011'     ; 63 - ?
                retlw           b'11111111'     ;(64 - @)

                retlw           b'11111010'     ; 65 - A .-
                retlw           b'11100111'     ; 66 - B -...
                retlw           b'11100101'     ; 67 - C -.-.
                retlw           b'11110011'     ; 68 - D -..
                retlw           b'11111101'     ; 69 - E .
                retlw           b'11101101'     ; 70 - F ..-.
                retlw           b'11110001'     ; 71 - G --.
                retlw           b'11101111'     ; 72 - H ....
                retlw           b'11111011'     ; 73 - I ..
                retlw           b'11101000'     ; 74 - J .---
                retlw           b'11110010'     ; 75 - K -.-
                retlw           b'11101011'     ; 76 - L .-..
                retlw           b'11111000'     ; 77 - M --
                retlw           b'11111001'     ; 78 - N -.
                retlw           b'11110000'     ; 79 - O ---
                retlw           b'11101001'     ; 80 - P .--.
                retlw           b'11100010'     ; 81 - Q --.-
                retlw           b'11110101'     ; 82 - R .-.
                retlw           b'11110111'     ; 83 - S ...
                retlw           b'11111100'     ; 84 - T -
                retlw           b'11110110'     ; 85 - U ..-
                retlw           b'11101110'     ; 86 - V ...-
                retlw           b'11110100'     ; 87 - W .--
                retlw           b'11100110'     ; 88 - X -..-
                retlw           b'11100100'     ; 89 - Y -.--
                retlw           b'11100011'     ; 90 - Z --..

;--[ table for converting the DTMF numbers from the decoder to a  ]--:
;--[ better format... the pins are a little mixed up ]--:

DTMFTabl
                addwf   PCL, F

                retlw   0x0D
                retlw   0x08
                retlw   0x04 
                retlw   0x0F ; #
                retlw   0x02 

                retlw   0x00
                retlw   0x06
                retlw   0x0B
                retlw   0x01
                retlw   0x09

                retlw   0x05
                retlw   0x0A
                retlw   0x03
                retlw   0x0E ; *
                retlw   0x07

                retlw   0x0C

;--[ list of the closing minutes ]--:
SulkuTabl
                addwf   PCL, F
                retlw   0x05            ;5min
                retlw   0x0A            ;10min
                retlw   0x14            ;20
                retlw   0x1E            ;30
                retlw   0x28            ;40
                retlw   0x32            ;50
                retlw   0x3C            ;60
                retlw   0x78            ;120
                retlw   0xF0            ;240

                retlw   0x01            ;rfu
                retlw   0x01
                retlw   0x01
                retlw   0x01
                retlw   0x01
                retlw   0x01

;---------------------------------------------------------------------------:

;--[ beginning of the code ]--:
Alku:
        movlw   0x07 
        movwf   CMCON           ; compare off (16F628)

        bcf     INTCON, 2       ; Clear TMR0 Interrupt Flag
        bsf     INTCON, 7       ; Global Interrupts on
        bsf     INTCON, 5       ; TMR0 Interrupt on
        clrf    TMR0            ; Clear TMR0
        clrwdt

        movlw   b'11000011'     ; Interrupt comes every 16. cycles
        bsf     STATUS, RP0     ; bank1
        movwf   OPTION_REG^80H
        bcf     STATUS, RP0     ; bank0
        clrf    TMR0            ; Clear TMR0 again.

        bsf     STATUS, RP0     ; bank1
        clrf    TRISA^80H       ; PORTA to outputs
        movlw   b'10111111'     ; PORTB to inputs, except one
        movwf   TRISB^80H
        bcf     STATUS, RP0     ; bank0

        bcf     PORTA, IO_TX    ; turn the transmitter off
        bsf     PORTA, IO_Mute  ; mute off
        bsf     PORTB, IO_Pin12 ; RB6 pin on

        movlw   0x1A
        call    LueEEPROM      ; get the max talktime from the EEPROM
        movwf   CloseTime       

        movlw   0x1B
        call    LueEEPROM
        movwf   IDTime

        movlw   0x1C
        call    LueEEPROM
        movwf   CWPitch

        movlw   0x1D
        call    LueEEPROM      ; get the CWSpeed from the EEPROM
        movwf   CWSpeed

        movlw   0x1E
        call    LueEEPROM
        movwf   SpTimeOut

        movlw   0x1F
        call    LueEEPROM
        movwf   SpTimeOutL

;--[ here we will loop forever ]--:
MainLoop:
        bcf     PORTA, IO_TX    ; transmitter off
        btfss   PORTB, IO_Sql   ; wait for the squelch to open (RB5)
        goto    MainLoop

;--[ squelch opened ]--:
SalpaLoop:
        btfss   PORTB, IO_Tone  ; Tone was heard?? (RB7)
        goto    Piippa

        btfsc   PORTB, IO_DTMF  ; DTMF was heard?
        goto    OnkoDTMFAani

        btfss   PORTB, IO_Sql   ; If the squelch is close, return to the beginning
        goto    MainLoop        ; If it's open, then keep looping..
        goto    SalpaLoop

OnkoDTMFAani
        movfw   PORTB
OdotaLoppua
        btfsc   PORTB, IO_DTMF  ; wait for the tone to end
        goto    OdotaLoppua

        andlw   0x0F
        call    DTMFTabl
        movwf   Temp3

        xorlw   0x0E            ; 0x0E = STAR (*)!
        btfsc   STATUS, Z         
        goto    Piippa2         ; DTMF * was recognized, open the repeater.

        goto    SalpaLoop

DTMFIDAvaus:
        movfw   PORTB
        andlw   0x0F
        call    DTMFTabl

        xorlw   0x00            ; zero?
        btfss   STATUS, Z
        goto    Piippa2

AnnaInfo:
        btfsc   PORTB, IO_Sql   ; wait for the squelch to close
        goto    AnnaInfo

        bcf     PORTA, IO_Mute  ; Mute on
        bsf     PORTA, IO_TX    ; Transmitter on

        call    PSekTauko       ; a half second pause

;--[ send info ]--:           ; give the CW infotext
        movlw   0x20
        movwf   Temp3
        call    PSekTauko
SendInf
        bcf     STATUS, RP0
        movfw   Temp3
        call    LueEEPROM
        call    CW

        incf    Temp3, F
        movfw   Temp3

        xorlw   0x3B
        btfss   STATUS, Z
        goto    SendInf

        goto    ToistinAuki2

;--[ Tone was detected, wait for it to end here ]-:
Piippa:
        clrf    ToneCount       ; clear the ToneCount counter
Piippac:
        clrw                    ; pause
        call    KeepPause 
        clrw                    ; pause
        call    KeepPause 

        incf    ToneCount, F    ; increase the counter
        btfsc   PORTB, IO_Tone  ; if the tone ends now ---
        goto    MainLoop        ; --- go to the beginning!

        movfw   ToneCount       ; if the tone was heard long enough
        xorlw   0xA0   ;<--time ; then wait for the squelch to close
        btfsc   STATUS, Z       ; and open the repeater
        goto    Piippa2
        goto    Piippac

Piippa2:
        btfsc   PORTB, IO_DTMF  ; DTMF was heard?
        goto    DTMFIDAvaus     ; check if it's zero?
                                ; if it is, give the CW info text

        btfsc   PORTB, IO_Sql   ; Wait for the squelch to close ;C
        goto    Piippa2
        goto    ToistinAuki     ; open the repeater

;--[ REPEATER IS OPEN NOW ]--:
ToistinAuki:
        bcf     PORTA, IO_Mute   ; Mute on

        bsf     PORTA, IO_TX     ; Transmitter on!
        call    PSekTauko        ; a half second pause

;--[ Send ID1 ]--:             ; gives the repeater callsign in CW
        clrf    Temp3
SendID
        bcf     STATUS, RP0
        movfw   Temp3
        call    LueEEPROM
        call    CW

        incf    Temp3, F
        movfw   Temp3

        xorlw   0x08
        btfss   STATUS, Z
        goto    SendID

ToistinAuki2:

        movlw   0x3D
        call    LueEEPROM
        bsf     STATUS, RP0     ; bank0
        incf    EEDATA, F
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

        bsf     STATUS, RP0     ; bank1
        movfw   EEDATA
        bcf     STATUS, RP0     ; bank0
        xorlw   0x00
        btfss   STATUS, Z
        goto    ToistinAuki3 

        movlw   0x3C
        call    LueEEPROM
        bsf     STATUS, RP0     ; bank0
        incf    EEDATA, F
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

ToistinAuki3:

        call    PSekTauko       ; a half second pause
        call    Bongo           ; make a sound

        clrf    Vuorot          ; clear the turn counter
        clrf    IntrCount       ; clear IntrCount
        clrf    Counter         ; clear the counter
        clrf    Counter_L       ; clear the big counter
        clrf    Counter_H

;--[ wait for someone to say something after the repeater opens ]--:
;--[ if no-one says anything in X seconds, the repeater will close ]--:

PLoop:
        btfsc   PORTB, IO_Sql
        goto    PLoopI          ; someone said something

        movfw   Counter
        xorwf   SpTimeOut, W
        btfsc   STATUS, Z
        goto    MainLoop        ; no-one wanted to talk, go to main
        goto    PLoop

PLoopI:
        clrf    IntrCount       ; clear IntrCount
        clrf    Counter         ; clear the counter

;--[ talking-loop ]--:
PuhuukoJoku:
        btfsc   PORTB, IO_Sql   ; is someone speaking?
        goto    JokuPuhuu	; yes

        movfw   Counter
        xorwf   SpTimeOut, W
        btfsc   STATUS, Z       ; if no-one is talking in X seconds, prepare
        goto    QSOLopetettu    ; to close the repeater
        goto    PuhuukoJoku

;--[ someone started talking ]--;
JokuPuhuu:
        incf    Vuorot, F       ; increase the amount of turns
        movfw   Vuorot
        xorlw   0x10            ; over 0x10?
        btfsc   STATUS, Z         
        decf    Vuorot, F

        bsf     PORTA, IO_TX    ; transmitter on, if it isn't on
        clrf    IntrCount       ; clear IntrCount
        clrf    Counter2        ; clear the counter

JokuPuhuuL1:
        bsf     PORTA, IO_Mute  ; Mute off
        movfw   CloseTime
        xorwf   Counter2, W     ; someone talked for too long?
        btfsc   STATUS, Z
        goto    Ympari          ; (CL!) close it !

        btfsc   PORTB, IO_DTMF  ; hearing and DTMF during a turn?
        goto    DTMFPainettu

        btfsc   PORTB, IO_Sql   ; still talking?
        goto    JokuPuhuuL1     ; yes = keep on looping

JokuPuhuuL2:                    ; the squelch closed, wait for half a second
        movlw   0x78            ; if it happens to come back? (mobile stations for example)
        movwf   IntrCount       ; IntrCount to half of the max value
        clrf    Counter         ; clear the counter
JokuPuhuuL3:
        btfss   PORTB, IO_Sql   ; did the squelch come back?
        goto    JokuPuhuuL4

        goto    JokuPuhuuL1     ; keep on talking

JokuPuhuuL4:
        movfw   Counter
        xorlw   0x01
        btfss   STATUS, Z
        goto    JokuPuhuuL3     ; wait

;--[ "OVER" ]---;

        movfw   IDTime          ; is IDTime greater than Counter_H?
        subwf   Counter_H, W
        btfss   STATUS, C
        goto    VuoroLoppui

        bcf     PORTA, IO_Mute   ; mute on

;--[ Send ID2 ]--:             ; sends the call of the repeater (every X minutes)
        movlw   0x08
        movwf   Temp3
SendID2
        bcf     STATUS, RP0
        movfw   Temp3
        call    LueEEPROM
        call    CW

        incf    Temp3, F
        movfw   Temp3

        xorlw   0x10
        btfss   STATUS, Z
        goto    SendID2

        call    PSekTauko        ; a half second pause
        clrf    Counter_L        ; clear the ID counters
        clrf    Counter_H        ; because we just gave out the ID!

VuoroLoppui

        call    Bongo           ; make a sound
        goto    PLoopI          ; return to the beginning

WaitForDTMF
        btfss   PORTB, IO_Sql   ; squelch?
        goto    VuoroLoppui
        btfss   PORTB, IO_DTMF  ; wait for the sound to begin
        goto    WaitForDTMF

        movfw   PORTB           ; PORTB -> W
        andlw   0x0F            ; mask.. we need only the first 4 bits
        call    DTMFTabl        ; call the DTMFTabl (result goes to W)
WaitForEnd
        btfsc   PORTB, IO_DTMF  ; wait for the DTMF tone to end
        goto    WaitForEnd
        return                

DTMFPainettu
        movfw   PORTB
        andlw   0x0F
        call    DTMFTabl
        movwf   Temp3

        xorlw   0x0F            ; # Pound sign pressed?
        btfsc   STATUS, Z         
        goto    DTMFKomento

        movfw   Temp3
        xorlw   0x0E            ; * Star pressed?
        btfsc   STATUS, Z         
        goto    DTMFKomento

        movfw   Temp3
        xorlw   0x0A            ; A pressed?
        btfsc   STATUS, Z         
        goto    Setuppi
        goto    JokuPuhuuL1

;--[ someone pressed the star (*) or the pound sign (#) .. ]--:
;--[ wait for the tone to end and then wait for the next tone ]--:
DTMFKomento:
        btfsc   PORTB, IO_DTMF  ; wait for the tone to end
        goto    DTMFKomento
DTMFKom1:

        btfss   PORTB, IO_Sql   ; squelch?
        goto    VuoroLoppui

        btfss   PORTB, IO_DTMF  ; wait for a tone to begin
        goto    DTMFKom1

        movfw   PORTB
        andlw   0x0F
        call    DTMFTabl

        movwf   Temp3
        xorlw   0x09            ; pressed nine? close the repeater
        btfsc   STATUS, Z
        goto    DTMFSulku

        ;--- here is a space for additional commands that begin with a * or #!

        ;movwf   Temp3
        ;xorlw   number
        ;btfsc   STATUS, Z
        ;goto    dosomething

        goto    JokuPuhuuL1
     
Setuppi:
        btfsc   PORTB, IO_DTMF  ; wait for the tone to end
        goto    Setuppi

        bcf     PORTA, IO_Mute   ; mute on that the code won't be repeated :)

        ; !!!!! PASSCODE 234 !!!!!
        call    WaitForDTMF
        xorlw   0x02            ;<-------- 2
        btfss   STATUS, Z
        goto    JokuPuhuuL1

        call    WaitForDTMF
        xorlw   0x03            ;<-------- 3
        btfss   STATUS, Z
        goto    JokuPuhuuL1

        call    WaitForDTMF
        xorlw   0x04            ;<-------- 4
        btfss   STATUS, Z
        goto    JokuPuhuuL1

SetL1   btfsc   PORTB, IO_Sql   ; wait for the squelch to close
        goto    SetL1
        bcf     PORTA, IO_Mute   ; mute on that the code won't be repeated :)

        call    PSekTauko
        movlw   'S'
        call    CW
        movlw   'E'
        call    CW
        movlw   'T'
        call    CW
        call    PSekTauko
        call    Bongo           

        clrf    Counter

;--[ THE SERVICE MODE ]--:
HuoltoTila

        movfw   Counter
        xorlw   0x0A            ; 0a = 10 seconds
        btfsc   STATUS, Z
        goto    JokuPuhuuL1
        
        btfss   PORTB, IO_DTMF  ; wait for a tone to begin
        goto    HuoltoTila

        movfw   PORTB
        andlw   0x0F
        call    DTMFTabl

Huolto2:
        btfsc   PORTB, IO_DTMF  ; wait for the tone to end
        goto    Huolto2
Huol1   btfsc   PORTB, IO_Sql   ; wait for the squelch to close
        goto    Huol1

        movwf   Temp3
        xorlw   0x0E            ; * pressed - return to the beginning
        btfsc   STATUS, Z
        goto    JokuPuhuuL1

        movfw   Temp3
        xorlw   0x0F            ; # pressed - go to the statistics thing
        btfsc   STATUS, Z
        goto    Tilasto

        movfw   Temp3
        xorlw   0x0A            ; RB6 off
        btfsc   STATUS, Z
        goto    RB6_Off

        movfw   Temp3
        xorlw   0x0B            ; RB6 on
        btfsc   STATUS, Z
        goto    RB6_On

        movfw   Temp3
        xorlw   0x09            ; close forever
        btfsc   STATUS, Z
        goto    SuljeLopullisesti

        bcf     PORTA, IO_Mute  ; mute on
        movfw   Temp3

        call    PSekTauko
        movfw   Temp3
        call    CW              ; give back the number and 'R'
        call    PSekTauko
        movlw   'R'
        call    CW
        call    PSekTauko
        bsf     PORTA, IO_Mute  ; mute off

        bcf     PORTA, IO_TX

        call    SulkuTabl
        movwf   Temp3

        clrf    Counter_L
        clrf    Counter_H

OdotaAika
        movfw   IntrCount       ; if the device has been closed,
        xorlw   0x10            ; flash the LED differently, so you know
        btfsc   STATUS, Z       ; that someone has closed it on purpose
        bsf     PORTA, IO_Led

        movfw   Temp3           ; is the closing time same than Counter_H?
        xorwf   Counter_H, W
        btfss   STATUS, Z
        goto    OdotaAika	; still wait

        bcf     PORTA, IO_Mute  ; mute on

        bsf     PORTA, IO_TX
        call    PSekTauko
        movlw   'Q'
        call    CW
        movlw   'R'
        call    CW
        movlw   'V'
        call    CW
        call    PSekTauko
        bcf     PORTA, IO_TX

        bsf     PORTA, IO_Mute   ; mute off, back to business!

        goto    MainLoop
      
RB6_On:
        bsf     PORTB, IO_Pin12
        bcf     PORTA, IO_Mute   ; mute on
        call    PSekTauko
        movlw   'O'
        call    CW
        movlw   'N'
        call    CW
        call    PSekTauko
        bsf     PORTA, IO_Mute   ; mute off
        goto    SetL1

RB6_Off:
        bcf     PORTB, IO_Pin12
        bcf     PORTA, IO_Mute   ; mute on
        call    PSekTauko
        movlw   'O'
        call    CW
        movlw   'F'
        call    CW
        movlw   'F'
        call    CW
        call    PSekTauko
        bsf     PORTA, IO_Mute   ; mute off
        goto    SetL1


SuljeLopullisesti:
        bcf     PORTA, IO_Mute   ; mute on
        call    PSekTauko
        movlw   'Q'
        call    CW
        movlw   'R'
        call    CW
        movlw   'T'
        call    CW
        call    PSekTauko
        bsf     PORTA, IO_Mute  ; mute off
EndLess
        movfw   IntrCount       ; if the device has been closed FOREVER,
        xorlw   0x10            ; flash the LED differently, so you know
        btfsc   STATUS, Z       ; that someone has closed it on purpose
        bsf     PORTA, IO_Led  
                               
        bcf     PORTA, IO_TX
        goto    EndLess

DTMFSulku:
        btfsc   PORTB, IO_Sql   ; wait for the squelch to close
        goto    DTMFSulku

        bcf     PORTA, IO_Mute  ; mute on
        call    PSekTauko
        movlw   'C'
        call    CW
        movlw   'L'
        call    CW
        call    PSekTauko
        bsf     PORTA, IO_Mute  ; mute off

        goto    MainLoop

Tilasto:
        bcf     PORTA, IO_Mute  ; mute on
        movlw   0x1A
        movwf   CWSpeed

        movlw   'O'
        call    CW
        movlw   'P'  ; 3c3d3e3f
        call    CW
        movlw   'N' 
        call    CW
        movlw   ' '
        call    CW

        clrf    ToneCount
Tilasto1:
        ; give eeprom 3C+3D number twice
        movlw   0x3C
        call    LueEEPROM
        movwf   TilastoTmp
        call    SendHi
        movfw   TilastoTmp
        call    SendLo

        movlw   0x3D
        call    LueEEPROM
        movwf   TilastoTmp
        call    SendHi
        movfw   TilastoTmp
        call    SendLo

        movlw   ' '
        call    CW

        incf    ToneCount, F
        movfw   ToneCount
        xorlw   0x02
        btfss   STATUS, Z
        goto    Tilasto1

        movlw   ' '
        call    CW
        movlw   '='
        call    CW
        movlw   ' '
        movlw   'Q'
        call    CW
        movlw   'S'
        call    CW
        movlw   'O'
        call    CW
        movlw   ' '
        call    CW

        clrf    ToneCount
Tilasto2:
        ; give eeprom 3E+3F number twice
        movlw   0x3E
        call    LueEEPROM
        movwf   TilastoTmp
        call    SendHi
        movfw   TilastoTmp
        call    SendLo
        movlw   0x3F
        call    LueEEPROM
        movwf   TilastoTmp
        call    SendHi
        movfw   TilastoTmp
        call    SendLo
        movlw   ' '
        call    CW

        incf    ToneCount, F
        movfw   ToneCount
        xorlw   0x02
        btfss   STATUS, Z
        goto    Tilasto2

        movlw   ' '
        call    CW
        movlw   'K'
        call    CW

        bsf     PORTA, IO_Mute  ; mute off

        movlw   0x1D
        call    LueEEPROM      ; get CWSpeed from the EEPROM
        movwf   CWSpeed

        clrf    Counter
        clrf    IntrCount
TilastoL

        movfw   Counter
        xorlw   0x0A            ; 0a = 10 seconds
        btfsc   STATUS, Z
        goto    SetL1
        
        btfss   PORTB, IO_DTMF  ; wait for a tone to start
        goto    TilastoL

        movfw   PORTB
        andlw   0x0F
        call    DTMFTabl

Tilasto3:
        btfsc   PORTB, IO_DTMF  ; wait for the tone to end
        goto    Tilasto3
Tilasto4:
        btfsc   PORTB, IO_Sql   ; wait for the squelch to close
        goto    Tilasto4

        movwf   Temp3
        xorlw   0x0E            ; * pressed - go to the beginning
        btfsc   STATUS, Z
        goto    SetL1

        movfw   Temp3
        xorlw   0x0F            ; # pressed - go to the beginning
        btfss   STATUS, Z
        goto    SetL1

        bsf     STATUS, RP0     ; bank0
        movlw   0x3C            ; clear the counters
        movwf   EEADR
        clrf    EEDATA
        movwf   EEADR
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

        bsf     STATUS, RP0     ; bank0
        movlw   0x3D
        movwf   EEADR
        clrf    EEDATA
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

        bsf     STATUS, RP0     ; bank0
        movlw   0x3E
        movwf   EEADR
        clrf    EEDATA
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

        bsf     STATUS, RP0     ; bank0
        movlw   0x3F
        movwf   EEADR
        clrf    EEDATA
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

        bcf     PORTA, IO_Mute  ; mute on
        call    PSekTauko
        movlw   'R'             ; roger
        call    CW
        call    PSekTauko
        bsf     PORTA, IO_Mute  ; mute off

        goto    SetL1
      
SendHi:
        movwf   Temp3
        rrf     Temp3, F
        rrf     Temp3, F
        rrf     Temp3, F
        rrf     Temp3, W
        andlw   0x0F
        call    CW
        return

SendLo
        andlw   0x0F
        call    CW
        return


;--[ the repeater closes, but you still have X seconds to open it with a PTT press ]--:
;--[ after the X seconds it needs a tone or a * to open it again ]--:
QSOLopetettu:
        bcf     PORTA, IO_TX    ; transmitter off
        clrf    IntrCount
        clrf    Counter
QSOLop1:
        btfsc   PORTB, IO_Sql   ; will the squelch open?
        goto    JokuPuhuu

        movfw   Counter
        xorwf   SpTimeOutL, W
        btfsc   STATUS, Z       ; if no-one talks in X seconds then ...
        goto    QSOLoppui
        goto    QSOLop1

;--[ no one hasn't said anything for a while ]--:
;--[ if the QSO lasted for more than 5 overs, give the ending callsign  ]--:
QSOLoppui:
        movfw   Vuorot
        movwf   Temp3
        movlw   0x05            ; 5 overs
        subwf   Temp3,1
        btfss   STATUS, C
        goto    MainLoop

        bsf     PORTA, IO_TX    ; transmiter on
        bcf     PORTA, IO_Mute  ; mute on
        call    PSekTauko        ; a half second pause

;--[ lhet ID3 ]--:             ; gives the repeater callsign
        movlw   0x10
        movwf   Temp3
SendID3
        bcf     STATUS, RP0
        movfw   Temp3
        call    LueEEPROM
        call    CW

        incf    Temp3, F
        movfw   Temp3

        xorlw   0x18
        btfss   STATUS, Z
        goto    SendID3

        call    PSekTauko       ; a half second pause
                
        bcf     PORTA, IO_TX    ; transmitter off
        bsf     PORTA, IO_Mute  ; mute off

        movlw   0x3F
        call    LueEEPROM
        bsf     STATUS, RP0     ; bank0
        incf    EEDATA, F
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

        bsf     STATUS, RP0     ; bank0
        movfw   EEDATA
        bcf     STATUS, RP0     ; bank0
        xorlw   0x00
        btfss   STATUS, Z
        goto    SendID4

        movlw   0x3E
        call    LueEEPROM
        bsf     STATUS, RP0     ; bank0
        incf    EEDATA, F
        bcf     STATUS, RP0     ; bank0
        call    KirjoitaEEPROM
        bcf     STATUS, RP0     ; bank0

SendID4:
        goto    MainLoop        ; wait for the next victim

;--[ someone talked too long ]---;
;--[ tell the people, that the repeater will close now! send 'CL' in CW for example ]--:
Ympari:
        movlw   0x18           
        call    LueEEPROM       ; first closing char
        call    CW

        movlw   0x19
        call    LueEEPROM       ; second closing char
        call    CW

        call    PSekTauko       ; pause
        bcf     PORTA, IO_TX    ; transmitter off

;--[ wait for the turn to end ]--;
Ympari1:
        btfsc   PORTB, IO_Sql   ; squelch still open?
        goto    Ympari1         ; if it is, then keep on looping here

        movlw   0x78          
        movwf   IntrCount       ; IntrCount to half of the maximum value
        clrf    Counter         ; clear the counter
Ympari2:
        btfsc   PORTB, IO_Sql   ; did the squelch come back?
        goto    Ympari1         ; keep on talking

        movfw   Counter
        xorlw   0x01
        btfss   STATUS, Z
        goto    Ympari2

        bsf     PORTA, IO_TX    ; transmitter on
        bcf     PORTA, IO_Mute  ; mute on
        call    PSekTauko       ; keep a pause, then go to "QSOLopetettu"
        call    Bongo
        bcf     PORTA, IO_Mute  ; mute on again
        call    Bongo           ; make a double sound to indicate, that the person does not talk anymore
        call    PSekTauko       ; a half second pause
        bcf     PORTA, IO_TX    ; transmitter off

        goto    QSOLopetettu

;--[ send the CW char in the register W ]--:
CW:
        call    CWMerkki
        movwf   CWChar

        xorlw   0xFF            ; all 1?
        btfsc   STATUS, Z
        return                  ; skip the char

        movfw   CWChar
        xorlw   0x00            ; all 0?
        btfsc   STATUS, Z
        goto    TyhjaMerkki

        movlw   7
        movwf   Temp2

EtsiNolla:
        rlf     CWChar, F
        btfss   STATUS, C
        goto    NollaLoytyi
        decfsz  Temp2, F
        goto    EtsiNolla

NollaLoytyi:
        rlf     CWChar, F
        btfss   STATUS, C
        call    Viiva             ; if zero, sound lasts for 2 units
        call    SendUnitOn        ; sound for 1 unit
        call    SendUnitOff       ; 1 unit pause
        decfsz  Temp2, F
        goto    NollaLoytyi

        call    SendUnitOff       ; 2 units pause
        call    SendUnitOff       
        return
                
Viiva:
        call    SendUnitOn  
        call    SendUnitOn
        return

;--[ sends one unit... a dash is 3 units ]--:
SendUnitOn      movfw   CWSpeed
                movwf   Temp
SendUnitOn1
                bsf     PORTA, IO_SND   ; sound out
                movlw   0x9f
                call    KeepPause       ; keep a pause
                movfw   CWPitch
                call    KeepPause       ; keep a pause

                bcf     PORTA, IO_SND   ; no sound out
                movlw   0x9f
                call    KeepPause       ; keep a pause
                movfw   CWPitch
                call    KeepPause       ; keep a pause

                decfsz  Temp, F         ; decrease Temp until zero
                goto    SendUnitOn1
                return

;--[ keeps a one unit pause... a dash is 3 units ]--:
SendUnitOff     movfw   CWSpeed
                movwf   Temp
SendUnitOff1
                bcf     PORTA, IO_SND   ; NO sound out
                movlw   0x9f
                call    KeepPause       ; keep a pause
                movfw   CWPitch
                call    KeepPause       ; keep a pause

                bcf     PORTA, IO_SND   ; NO sound out
                movlw   0x9f
                call    KeepPause       ; keep a pause
                movfw   CWPitch
                call    KeepPause       ; keep a pause

                decfsz  Temp, F         ; decrease Temp until zero
                goto    SendUnitOff1
                return

TyhjaMerkki:
                call    SendUnitOff
                call    SendUnitOff
                call    SendUnitOff
                call    SendUnitOff
                call    SendUnitOff
                call    SendUnitOff
                call    SendUnitOff
                return        

;--[ someone's KeepPause -routine ]--:
KeepPause       addlw   0x01                ;adds one to reg 1
                btfss   STATUS, Z           ;if z flag is on, skip next
                goto    KeepPause           ;z is off? ok lets inc W until
                return                      ;it goes over 255

PSekTauko:
                movlw   0x78
                movwf   IntrCount       ; IntrCount to half of the value
                clrf    Counter         ; clear the counter
PSekLoop:
                movfw   Counter
                xorlw   0x01
                btfss   STATUS, Z
                goto    PSekLoop
                return

Bongo:
                bcf     PORTA, IO_Mute  ; mute on
                movlw   0x16            ; <- length of the first tone
                movwf   Temp
SendBongo1                              ;-higher sound
                bsf     PORTA, IO_SND   ; sound out
                movlw   0x3f
                call    KeepPause       ; keep a pause
                movlw   0xFF            ; <- the frequency of the first tone
                call    KeepPause       ; keep a pause

                bcf     PORTA, IO_SND   ; no sound out
                movlw   0x3f
                call    KeepPause       ; keep a pause
                movlw   0xFF            ; <- the frequency of the first tone
                call    KeepPause       ; keep a pause

                decfsz  Temp, F         ; decrease Temp until zero
                goto    SendBongo1

                movlw   0x16            ; <- length of the second tone
                movwf   Temp
SendBongo2                              ;-lower sound after the higher sound there
                bsf     PORTA, IO_SND   ; sound out
                movlw   0x1f
                call    KeepPause       ; keep a pause
                movlw   0xFF            ; <- the frequency of the second tone
                call    KeepPause       ; keep a pause

                bcf     PORTA, IO_SND   ; no sound out
                movlw   0x1f
                call    KeepPause       ; keep a pause
                movlw   0xFF            ; <- the frequency of the second tone
                call    KeepPause       ; keep a pause

                decfsz  Temp, F         ; decrease Temp until zero
                goto    SendBongo2

                bsf     PORTA, IO_Mute  ; mute off
                return

LueEEPROM:
                bsf     STATUS, RP0     ; bank1
                movwf   EEADR
                bsf     EECON1, RD
                movf    EEDATA, W
                bcf     STATUS, RP0
                return

KirjoitaEEPROM:
                bsf     STATUS, RP0   ; Bank 1
                bcf     INTCON, GIE   ; Disable INTs.
                bsf     EECON1, WREN ; Enable Write
                movlw   0x55 
                movwf   EECON2    ; Write 55h
                movlw   0xAA
                movwf   EECON2    ; Write AAh
                bsf     EECON1,WR ; Set WR bit
                                      ; begin write
EELoop          btfsc   EECON1, WR ; wait for the byte to be saved
                goto    EELoop
                bcf     EECON1, WREN
                bcf     EECON1, EEIF
                bsf     INTCON, GIE   ; Enable INTs.
                bcf     STATUS, RP0   ; Bank 0
                return

        org     0x03E8
        data    'V','e','r','.','2','.','0',' '
        data    '1','6','-','0','1','-','0','4'
        data    '*','O','H','3','H','R','K','*'

;--[ the end ]--:
        end
